@@ -34,12 +34,7 @@
 #include <pwd.h>
 #include <dirent.h>
 
-#ifdef linux
-# include <linux/cdrom.h>
-# include "linux-cdrom.h"
-#else
-# error TCD currently only builds under Linux systems.
-#endif
+#include "linux-cdrom.h"
 
 #include "cddb.h"
 
@@ -67,7 +62,7 @@ static int num_digits( int num)
 int tcd_readcddb( cd_struct* cd, char* filename )
 {
 	FILE* fp;
-	char string[100];
+	char string[256];
 	int trk;	
 	
 	fp = fopen( filename, "r" );
@@ -78,7 +73,7 @@ int tcd_readcddb( cd_struct* cd, char* filename )
 		return -1;
 	
 	/* AC: dont feof.. feof is only true _after_ eof is read */	
-	while(fgets( string, 80, fp )!=NULL)
+	while(fgets( string, 255, fp )!=NULL)
 	{
 		string[strlen(string)-1] = 0;
 		/* If it's a comment, ignore. */
@@ -199,7 +194,7 @@ void tcd_call_cddb_slave(cd_struct *cd, char *package, char *version)
     }
 
     fprintf(fp, "%s", buf);
-    fprintf(fp, "client %s %s %d\n", package, version, getpid());
+    fprintf(fp, "client %s %s %d %d\n", package, version, getpid(), cd->cd_dev);
     pclose(fp);
 }
 
