@@ -24,8 +24,6 @@
 */
 
 #include <config.h>
-#include <gnome.h>
-#include <gdk/gdkkeysyms.h>
 
 #include <fcntl.h>
 #include <stdio.h>
@@ -41,12 +39,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifdef linux
-# include <linux/cdrom.h>
-# include "linux-cdrom.h"
-#else
-# error TCD currently only builds under Linux systems.
-#endif
+#include <gnome.h>
+#include <gdk/gdkkeysyms.h>
+
+#include "linux-cdrom.h"
 
 #include "gtracked.h"
 #include "prefs.h"
@@ -60,18 +56,18 @@ enum { TIME_FIRST=-1, TRACK_E, TRACK_R, DISC_E, DISC_R, TIME_LAST };
 
 char *display_types[] = 
 {
-    "trk-e",
-    "trk-r",
-    "dsc-e",
-    "dsc-r"
+    N_("trk-e"),
+    N_("trk-r"),
+    N_("dsc-e"),
+    N_("dsc-r")
 };
 
 char *play_types[] = 
 {
-    "loop-cd",
-    "loop-t",
-    "normal",
-    "random"
+    N_("loop-cd"),
+    N_("loop-t"),
+    N_("normal"),
+    N_("random")
 };
 
 /* globals */
@@ -124,9 +120,9 @@ void setup_time_display(GtkWidget *table);
 void setup_fonts(void);
 void init_window(void);
 GtkWidget* make_button_with_pixmap(char *pic, GtkSignalFunc func,
-				   guint key,  gchar *tooltip);
+				   Shortcut *key,  gchar *tooltip);
 GtkWidget* make_button_stock(char *stock, GtkSignalFunc func,
-			     guint key, gchar *tooltip);
+			     Shortcut *key, gchar *tooltip);
 void reload_info(int signal);
 void exit_action(void);
 void start_action(void);
@@ -159,10 +155,8 @@ int skip_cb(GtkWidget *widget, GdkEvent *event, gpointer *data)
 	if(release_t)
 	{
 	    gtk_timeout_remove(release_t);
-	    switch(GPOINTER_TO_INT(data))
+	    if(GPOINTER_TO_INT(data) > 0)
 	    {
-		/* +track */
-	    case 2:
 		if(cd.cur_t < cd.last_t)
 		{   
 		    cd.cur_t++;
@@ -170,9 +164,9 @@ int skip_cb(GtkWidget *widget, GdkEvent *event, gpointer *data)
 		    if(cd.play_method==REPEAT_TRK)
 			cd.repeat_track = cd.cur_t;
 		}
-		break;
-		/* -track */
-	    case -2:
+	    }
+	    else
+	    {
 		if( cd.cur_t > cd.first_t )
 		{
 		    if( (cd.t_sec+(cd.t_min*60)) < 10 )
@@ -182,7 +176,6 @@ int skip_cb(GtkWidget *widget, GdkEvent *event, gpointer *data)
 		    if( cd.play_method==REPEAT_TRK )
 			cd.repeat_track = cd.cur_t;
 		}
-		break;
 	    }
 	}   
 	if(roll_t)
@@ -196,7 +189,7 @@ int skip_cb(GtkWidget *widget, GdkEvent *event, gpointer *data)
 }
 
 GtkWidget* make_button_with_pixmap(char *pic, GtkSignalFunc func,
-				   guint key ,  gchar *tooltip)
+				   Shortcut *key,  gchar *tooltip)
 {
     GtkWidget *button;
     GtkWidget *pixmap;
@@ -214,7 +207,7 @@ GtkWidget* make_button_with_pixmap(char *pic, GtkSignalFunc func,
     button = gtk_button_new();
     gtk_container_add( GTK_CONTAINER(button), pixmap );
 
-    if(key != -1)
+    if(key)
 	add_key_binding(button, "clicked", tooltip, key);
 
     if(func)
@@ -227,7 +220,7 @@ GtkWidget* make_button_with_pixmap(char *pic, GtkSignalFunc func,
 }	                        
 
 GtkWidget* make_button_stock(char *stock, GtkSignalFunc func,
-			     guint key, gchar *tooltip)
+			     Shortcut *key, gchar *tooltip)
 {
     GtkWidget *button;
     GtkWidget *pixmap;
@@ -236,7 +229,7 @@ GtkWidget* make_button_stock(char *stock, GtkSignalFunc func,
     button = gtk_button_new();
     gtk_container_add(GTK_CONTAINER(button), pixmap);
 
-    if(key != -1)
+    if(key)
 	add_key_binding(button, "clicked", tooltip, key);
 
     if(func)
@@ -300,9 +293,9 @@ GtkWidget *make_small_buttons(void)
     
     table = gtk_table_new(TRUE, 6,1);
 
-    b1 = make_button_stock(GNOME_STOCK_PIXMAP_PROPERTIES, edit_window, 'T', _("Open track editor"));
-    b2 = make_button_stock(GNOME_STOCK_PIXMAP_PREFERENCES, preferences, -1, _("Preferences"));
-    b3 = make_button_stock(GNOME_STOCK_PIXMAP_QUIT, quit_cb, 'Q', _("Quit"));
+    b1 = make_button_stock(GNOME_STOCK_PIXMAP_PROPERTIES, edit_window, &prefs.tracked, _("Open track editor"));
+    b2 = make_button_stock(GNOME_STOCK_PIXMAP_PREFERENCES, preferences, NULL, _("Preferences"));
+    b3 = make_button_stock(GNOME_STOCK_PIXMAP_QUIT, quit_cb, &prefs.quit, _("Quit"));
 
     gtk_table_attach_defaults(GTK_TABLE(table), b1, 0, 2, 0, 1);
     gtk_table_attach_defaults(GTK_TABLE(table), b2, 2, 4, 0, 1);
@@ -323,28 +316,28 @@ GtkWidget* create_buttons(void)
 
 /* TOP ROW */ 
      
-    playbutton = make_button_with_pixmap("play", NULL, 'P', _("Play"));
+    playbutton = make_button_with_pixmap("play", NULL, &prefs.play, _("Play/Pause"));
     status_changed();
     b1 = playbutton;
     
-    b2 = make_button_with_pixmap( "stop", stop_cb, 'S', _("Stop") );
-    b3 = make_button_with_pixmap( "eject", eject_cb, 'E', _("Eject") );
+    b2 = make_button_with_pixmap( "stop", stop_cb, &prefs.stop, _("Stop") );
+    b3 = make_button_with_pixmap( "eject", eject_cb, &prefs.eject, _("Eject") );
     
     gtk_table_attach_defaults(GTK_TABLE(table), b1, 0, 1, 0, 1);
     gtk_table_attach_defaults(GTK_TABLE(table), b2, 1, 2, 0, 1);
     gtk_table_attach_defaults(GTK_TABLE(table), b3, 2, 3, 0, 1);
 
 /* MIDDLE ROW */
-    rw = make_button_with_pixmap("rw", NULL, -1, _("Skip backwards"));
-    ff = make_button_with_pixmap("ff", NULL, -1, _("Skip forwards"));
+    rw = make_button_with_pixmap("rw", NULL, NULL, _("Skip backwards"));
+    ff = make_button_with_pixmap("ff", NULL, NULL, _("Skip forwards"));
 
     gtk_widget_set_events(rw, GDK_BUTTON_PRESS_MASK
 			  | GDK_BUTTON_RELEASE_MASK);
     gtk_widget_set_events(ff, GDK_BUTTON_PRESS_MASK
 			  | GDK_BUTTON_RELEASE_MASK);
 
-    add_key_binding(rw, "clicked", _("Skip backwards"), '-');
-    add_key_binding(rw, "clicked", _("Skip forwards"), '+');
+    add_key_binding(rw, "clicked", _("Skip backwards"), &prefs.back);
+    add_key_binding(rw, "clicked", _("Skip forwards"), &prefs.forward);
 
     gtk_signal_connect(GTK_OBJECT(ff), "event",
 		       GTK_SIGNAL_FUNC(skip_cb), GINT_TO_POINTER(2));
@@ -373,7 +366,7 @@ void setup_colors( void )
     
     colormap = gtk_widget_get_colormap(status_area);
     
-    gdk_color_parse("#555555", &darkgrey);
+    gdk_color_parse("#888888", &darkgrey);
     gdk_color_alloc(colormap, &darkgrey);
 
     g_snprintf(tmp, 15, "#%02X%02X%02X", 
@@ -431,13 +424,16 @@ void draw_titles(void)
     gdk_gc_set_foreground( gc, &track_color );
     inc = (tfont->ascent+tfont->descent)-2;
 
-    gdk_draw_text(status_db,tfont,gc,4,39, cd.artist, 
+    gdk_draw_text(status_db,tfont,gc,4,39, cd.artist,
 		  strlen(cd.artist));
     gdk_draw_text(status_db,tfont,gc,4,39+inc, cd.album, 
 		  strlen(cd.album));
     gdk_draw_text(status_db,tfont,gc,4,39+inc+inc, cd.trk[cd.cur_t].name, 
 		  strlen(cd.trk[cd.cur_t].name));
-    gtk_window_set_title( GTK_WINDOW(window), cd.trk[cd.cur_t].name );
+    if (cd.cur_t == 0)
+      gtk_window_set_title(GTK_WINDOW(window), cd.dtitle);
+    else
+      gtk_window_set_title(GTK_WINDOW(window), cd.trk[cd.cur_t].name);
 }
 
 void draw_time_scanning(void)
@@ -505,13 +501,15 @@ void draw_status(void)
 	}
 	if(cd.isplayable)
 	{		
+	    char *dtmsg = gettext(display_types[prefs.time_display]);
+	    char *pmsg =  gettext(play_types[cd.play_method]);
 	    gdk_gc_set_foreground(gc, &darkgrey);
 	    gdk_draw_text( status_db,sfont,gc,48,26, 
-			   display_types[prefs.time_display], 
-			   strlen(display_types[prefs.time_display]));
+	    		   dtmsg, 
+			   strlen(dtmsg));
 	    gdk_draw_text( status_db,sfont,gc,2,26, 
-			   play_types[cd.play_method], 
-			   strlen(play_types[cd.play_method]));
+			   pmsg,
+			   strlen(pmsg));
 	}
     }
     else
@@ -519,6 +517,7 @@ void draw_status(void)
 	strncpy(tmp, cd.errmsg, sizeof(tmp));
 	gdk_draw_text(status_db,tfont,gc,
 		      4, 39, tmp, strlen(tmp));
+	gtk_window_set_title(GTK_WINDOW(window), PACKAGE" "VERSION" ");
     }
 
     /* Finally, update the display */
@@ -555,22 +554,43 @@ void adjust_status(void)
 gint slow_timer( gpointer *data )
 {
     if( cd.cddb_id != cur_goto_id )
-	make_goto_menu();
-
-    if( cd.err || !cd.isplayable )
     {
-	tcd_readtoc(&cd);
-	tcd_readdiskinfo(&cd);
+	make_goto_menu();
+	update_editor();
     }
-
     if( cd.sc.cdsc_audiostatus != CDROM_AUDIO_PLAY &&
 	cd.sc.cdsc_audiostatus != CDROM_AUDIO_PAUSED )
     {
 	if( cd.play_method == REPEAT_CD )
 	    tcd_playtracks( &cd, cd.first_t, cd.last_t );
     }
-    tcd_gettime(&cd);
-    adjust_status();
+    if(cd.isplayable) 
+    {
+	tcd_gettime(&cd);
+	adjust_status();
+
+	if(cd.err) 
+	{
+	    cd.isplayable = FALSE;
+	    cd.isdisk = FALSE;
+	    cd.play_method = NORMAL;
+	    cd.repeat_track = -1;
+	}
+    }
+    else
+    {
+	tcd_readtoc(&cd);
+	if(!cd.err)
+	    tcd_readdiskinfo(&cd);
+	else
+	{
+	    cd.isplayable = TRUE;
+	    cd.isdisk = TRUE;
+	    cd.play_method = NORMAL;
+	    cd.repeat_track = -1;    
+	    update_editor();
+	}
+    }
     draw_status(); 
     return 1;
 }
@@ -643,9 +663,9 @@ void make_goto_menu()
 	item = gtk_menu_item_new_with_label(buf);
 	gtk_widget_show(item);
 	gtk_menu_append(GTK_MENU(gotomenu), item);
-	gtk_signal_connect_object(GTK_OBJECT(item), "activate",
-				  GTK_SIGNAL_FUNC(goto_track_cb), GINT_TO_POINTER(i));
-    }
+	gtk_signal_connect(GTK_OBJECT(item), "activate",
+			   GTK_SIGNAL_FUNC(goto_track_cb), GINT_TO_POINTER(i));
+   }
     if(gotoi) gtk_signal_disconnect(GTK_OBJECT(gotobutton), gotoi);
     gotoi = gtk_signal_connect_object(GTK_OBJECT(gotobutton), "event",
 				      GTK_SIGNAL_FUNC(button_press), GTK_OBJECT(gotomenu));
@@ -655,7 +675,15 @@ void make_goto_menu()
 
 gint fast_timer( gpointer *data )
 {
-    tcd_gettime(&cd);
+    if( cd.isplayable ) {
+	tcd_gettime(&cd);
+	if( cd.err ) {
+/*	   cd.isplayable = FALSE;
+	   cd.isdisk = FALSE;
+	   cd.play_method = NORMAL;
+	   cd.repeat_track = -1;*/
+	}
+    }
     status_changed();
     if((cd.play_method==REPEAT_TRK) && (cd.cur_t != cd.repeat_track))
 	tcd_playtracks(&cd, cd.repeat_track, -1);
@@ -808,7 +836,7 @@ void setup_time_display(GtkWidget *table)
 
 void setup_fonts(void)
 {
-    if( tfont )
+    if(tfont)
 	gdk_font_unref(tfont);
 
     if(prefs.trackfont)
@@ -827,7 +855,7 @@ void setup_fonts(void)
 void init_window(void)
 {
     setup_fonts();
-
+    /* Window */
     window = gnome_app_new("gtcd", "TCD 2.0");
     gtk_window_set_title(GTK_WINDOW(window), PACKAGE" "VERSION" ");
     gtk_window_set_wmclass( GTK_WINDOW(window), "main_window","gtcd");
@@ -836,13 +864,15 @@ void init_window(void)
     gtk_signal_connect(GTK_OBJECT(window), "delete_event",
 		       GTK_SIGNAL_FUNC(quit_cb), NULL);
 
-    gtk_container_border_width(GTK_CONTAINER(window), 3);
+    gtk_container_border_width(GTK_CONTAINER(window), GNOME_PAD_SMALL);
+
     gtk_widget_realize(window);
 
+    /* a gc */
     gc = gdk_gc_new(window->window);
     gdk_gc_copy(gc, window->style->white_gc);
 
-    main_box = gtk_hbox_new(FALSE, 3);
+    main_box = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
 
     tooltips = gtk_tooltips_new();
     if( prefs.tooltip )
@@ -859,9 +889,13 @@ void create_warning(char *message_text, char *type)
 
 void reload_info(int signal)
 {
-    tcd_close_disc(&cd);
-    tcd_init_disc(&cd, create_warning);
-    make_goto_menu();
+    tcd_post_init(&cd);
+    if(cd.isplayable)
+    {
+	make_goto_menu();
+	update_editor();
+    }
+
     draw_status();
 }
 
@@ -873,10 +907,17 @@ void exit_action(void)
 	tcd_stopcd(&cd);
 	break;
     case OpenTray:
+	cd.isdisk = TRUE;
 	tcd_ejectcd(&cd);
 	break;
     case CloseTray:
-	ioctl(cd.cd_dev, CDROMCLOSETRAY);
+	cd.isdisk = FALSE;
+	tcd_ejectcd(&cd);
+	if(cd.isplayable)
+	{
+	    make_goto_menu();
+	    update_editor();
+	}
 	break;
     case DoNothing:
     default:
@@ -886,12 +927,24 @@ void exit_action(void)
 
 void start_action(void)
 {
-    if(prefs.close_tray_on_start)
-	ioctl(cd.cd_dev, CDROMCLOSETRAY);
+    if(prefs.close_tray_on_start) {
+	cd.isdisk = FALSE;
+	tcd_ejectcd(&cd);
+	if(cd.isplayable)
+	{
+	    make_goto_menu();
+	    update_editor();
+	}
+    }
     switch(prefs.start_action)
     {
     case StartPlaying:
 	tcd_playtracks(&cd, cd.first_t, cd.last_t);
+	if(cd.isplayable)
+	{
+	    make_goto_menu();
+	    update_editor();
+	}
 	break;
     case StopPlaying:
 	tcd_stopcd(&cd);
@@ -913,12 +966,10 @@ int main (int argc, char *argv[])
     char *homedir;
     char rcfile[256];
 
-    argp_program_version = VERSION;
-
     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     textdomain(PACKAGE);
 
-    gnome_init("gtcd", NULL, argc, argv, 0, NULL);
+    gnome_init("gtcd", VERSION, argc, argv);
 
     homedir = getenv("HOME");
     g_snprintf(rcfile, 255, "%s/.tcd/gtcdrc", homedir);
