@@ -42,12 +42,11 @@
 #include <math.h>
 #include <sys/ioctl.h>
 
-#ifdef linux
-# include <linux/cdrom.h>
-# include "linux-cdrom.h"
-#else
-# error TCD currently only builds under Linux systems.
-#endif
+/* For _() */
+#include <config.h>
+#include <gnome.h>
+
+#include "linux-cdrom.h"
 
 #include "cddb.h"
 
@@ -88,23 +87,33 @@ int tcd_init_disc( cd_struct *cd, WarnFunc msg_cb )
 	    homedir = "/";                           
     }
     
-    if( !cd->err ) tcd_readtoc(cd);
-#ifdef DATABASE_SUPPORT
-    if( !cd->err ) tcd_readdiskinfo(cd);
-#endif
-    if( cd->err )
-    {
-	debug( "cdrom.c: tcd_init_disc exiting early (!cd->err)\n" );
-	return(-1);
-    }
-    
-#ifdef TCD_CHANGER_ENABLED
+#if defined(TCD_CHANGER_ENABLED)
     cd->nslots = ioctl( cd->cd_dev, CDROM_CHANGER_NSLOTS );
 #else
     cd->nslots = 0;
 #endif
     
     debug("cdrom.c: tcd_init_disc exiting normally\n" );
+    return( tcd_post_init( cd ));
+}
+
+int tcd_post_init( cd_struct *cd )
+{
+    debug("cdrom.c: tcd_post_init(%p) top\n", cd );
+
+    if( cd->isdisk && !cd->err ) {
+	tcd_readtoc(cd);
+#ifdef DATABASE_SUPPORT
+	tcd_readdiskinfo(cd);
+#endif
+	if( cd->err )
+	{
+	    debug( "cdrom.c: tcd_post_init exiting early (!cd->err)\n" );
+	    return(-1);
+	}
+    }
+
+    debug("cdrom.c: tcd_post_init exiting normally\n" );
     return(0);
 }
 
@@ -126,7 +135,7 @@ int tcd_readtoc( cd_struct *cd )
 	cd->isplayable=FALSE;
 	
 	tmp = ioctl( cd->cd_dev, CDROMREADTOCHDR, &cd->tochdr );
-	if( tmp < 0 )
+	if(tmp)
 	{
 		strcpy( cd->errmsg, "Can't read disc." );
 		cd->err = TRUE;
@@ -147,7 +156,7 @@ int tcd_readtoc( cd_struct *cd )
     	cd->trk[C(cd->last_t+1)].toc.cdte_track = CDROM_LEADOUT;
         cd->trk[C(cd->last_t+1)].toc.cdte_format = CDROM_MSF;
 	tmp = ioctl(cd->cd_dev, CDROMREADTOCENTRY, &cd->trk[C(cd->last_t+1)].toc );
-	if( tmp < 0 )
+	if(tmp)
 	{
 		strcpy( cd->errmsg, "Can't read disc." );
                 cd->err = TRUE;
@@ -163,7 +172,7 @@ int tcd_readtoc( cd_struct *cd )
                 cd->trk[C(i)].toc.cdte_format = CDROM_MSF;
 
                 tmp = ioctl( cd->cd_dev, CDROMREADTOCENTRY, &cd->trk[C(i)].toc );		
-                if( tmp < 0 )
+                if(tmp)
 		{
 			strcpy( cd->errmsg, "Can't read disc." );
                   	cd->err = TRUE;
@@ -276,12 +285,16 @@ int tcd_set_volume(cd_struct *cd, int volume)
 
 int tcd_get_volume(cd_struct *cd)
 {
+#ifdef CDROMVOLREAD
         struct cdrom_volctrl vol;
 
 	if(ioctl(cd->cd_dev, CDROMVOLREAD, &vol) < 0)
 		return -1;
 
 	return vol.channel0;
+#else
+	return 0;
+#endif
 }	
 	                                  
 int tcd_playtracks( cd_struct *cd, int start_t, int end_t )
@@ -291,10 +304,25 @@ int tcd_playtracks( cd_struct *cd, int start_t, int end_t )
 	int tmp;
 	debug("cdrom.c: tcd_playtracks( %p, %d, %d )\n", cd, start_t, end_t );
 	cd->err = FALSE;
-	cd->isplayable=FALSE;
-	cd->isdisk=FALSE;
 	
-	ioctl( cd->cd_dev, CDROMCLOSETRAY );
+	tcd_gettime(cd);
+	if(cd->err) 
+	{
+	    cd->isdisk = FALSE;
+	    tcd_ejectcd(cd);
+
+	    if(cd->err) 
+	    {
+
+		debug("cdrom.c: tcd_playtracks - error while fetching disc.\n");
+		return -1;
+
+	    }
+	}
+
+#ifdef CDROMCLOSETRAY
+	ioctl(cd->cd_dev, CDROMCLOSETRAY);
+#endif	        
 	
 	if(cd->trk[start_t].toc.cdte_ctrl == CDROM_DATA_TRACK)
 	    start_t++;		/* FIXME bad hack */
@@ -314,13 +342,28 @@ int tcd_playtracks( cd_struct *cd, int start_t, int end_t )
 	{
 		msf.cdmsf_min1 = cd->trk[end_t+1].toc.cdte_addr.msf.minute;
 		msf.cdmsf_sec1 = cd->trk[end_t+1].toc.cdte_addr.msf.second;
-		msf.cdmsf_frame1 = cd->trk[end_t+1].toc.cdte_addr.msf.frame;
+		msf.cdmsf_frame1 = cd->trk[end_t+1].toc.cdte_addr.msf.frame - 1;
+
+		if(msf.cdmsf_frame1 < 0)
+		{
+		    msf.cdmsf_sec1 += msf.cdmsf_frame1;
+		    msf.cdmsf_frame1 = 0;
+		}
+		if(msf.cdmsf_sec1 < 0)
+		{
+		    msf.cdmsf_min1 += msf.cdmsf_sec1;
+		    msf.cdmsf_sec1 = 0;
+		}
+		if(msf.cdmsf_min1 < 0)
+		{
+		    msf.cdmsf_min1 = 0;
+		}
 	}
 	msf.cdmsf_min1 += (msf.cdmsf_sec1 / 60);
 	msf.cdmsf_sec1 %= 60;
 	tmp = ioctl( cd->cd_dev, CDROMPLAYMSF, &msf );
 		
-	if( tmp < 0 )
+	if(tmp)
 	{
 		debug("cdrom.c: tcd_playtracks error. CDROMPLAYMSF ioctl error. Trying PLAYTRKIND\n" );
 		/* Try alternate method of playing  FIXME: handle -1 end_t */
@@ -330,7 +373,7 @@ int tcd_playtracks( cd_struct *cd, int start_t, int end_t )
 		trkind.cdti_ind1 = 0;      	/* end index */
 		                                
 		tmp = ioctl( cd->cd_dev, CDROMPLAYTRKIND, &trkind );
-		if( tmp < 0 )
+		if(tmp)
 		{
 			strcpy( cd->errmsg, "Error playing disc" );
 			cd->err = TRUE;
@@ -338,8 +381,6 @@ int tcd_playtracks( cd_struct *cd, int start_t, int end_t )
 			return -1;
 		}
 	}
-   	cd->isplayable=TRUE;                                                 
-	cd->isdisk=TRUE;
 
 	debug("cdrom.c: tcd_playtracks exiting normally\n" );
    	return tmp;
@@ -362,7 +403,23 @@ int tcd_play_seconds( cd_struct *cd, long int offset )
 	msf.cdmsf_frame0 = cd->sc.cdsc_absaddr.msf.frame;
 	msf.cdmsf_min1 = cd->trk[C(cd->last_t+1)].toc.cdte_addr.msf.minute;
 	msf.cdmsf_sec1 = cd->trk[C(cd->last_t+1)].toc.cdte_addr.msf.second;
-	msf.cdmsf_frame1 = cd->trk[C(cd->last_t+1)].toc.cdte_addr.msf.frame;
+	msf.cdmsf_frame1 = cd->trk[C(cd->last_t+1)].toc.cdte_addr.msf.frame - 1;
+
+	if(msf.cdmsf_frame1 < 0)
+	{
+	    msf.cdmsf_sec1 += msf.cdmsf_frame1;
+	    msf.cdmsf_frame1 = 0;
+	}
+	if(msf.cdmsf_sec1 < 0)
+	{
+	    msf.cdmsf_min1 += msf.cdmsf_sec1;
+	    msf.cdmsf_sec1 = 0;
+	}
+	if(msf.cdmsf_min1 < 0)
+	{
+	    msf.cdmsf_min1 = 0;
+	}
+	
 	if( msf.cdmsf_sec0 > 60 && (offset<0) )
 	{
 		msf.cdmsf_sec0 = 60-abs(offset);
@@ -370,7 +427,7 @@ int tcd_play_seconds( cd_struct *cd, long int offset )
 	}
 	
         tmp = ioctl( cd->cd_dev, CDROMPLAYMSF, &msf );
-   	if( tmp < 0 )
+   	if(tmp)
 	{
 		strcpy( cd->errmsg, "Error playing disc." );
                	cd->err = TRUE;
@@ -390,24 +447,37 @@ int tcd_ejectcd( cd_struct *cd )
 	int tmp;
 
 	debug("cdrom.c: tcd_eject(%p) top\n", cd );
-	tcd_stopcd(cd);
+	if( cd->isplayable ) tcd_stopcd(cd);
 	cd->err = FALSE;
 
-	tmp = ioctl( cd->cd_dev, CDROMEJECT );
-	/* Error? try injecting */
-	if( tmp < 0 )
+	if( cd->isdisk ) {
+
+		tmp = ioctl( cd->cd_dev, CDROMEJECT );
+		cd->isdisk = FALSE;
+		cd->isplayable = FALSE;
+		strcpy( cd->errmsg, "No disc in drive " );
+		cd->err = TRUE;
+
+	} else {
+#ifdef CDROMCLOSETRAY
 		tmp = ioctl( cd->cd_dev, CDROMCLOSETRAY );
+		cd->isdisk = TRUE;
+#endif
+		tmp = tcd_post_init( cd );
 
-   	if( tmp < 0 )
-	{
-		strcpy( cd->errmsg, "Can't eject disc." );
-               	cd->err = TRUE;
-
-		debug("cdrom.c: tcd_eject exiting early. eject error. %s\n", cd, 
+	   	if(tmp) {
+			strcpy( cd->errmsg, "Disc init error. " );
+               		cd->err = TRUE;
+			
+			debug("cdrom.c: tcd_eject - disc init error. %s\n",  
 			strerror(errno) );
 
-		return(-1);
-        }
+			return(-1);
+		}
+
+		cd->isdisk = TRUE;
+		cd->isplayable = TRUE;
+	}
 	cd->cur_t = 0;	
 	
 	debug("cdrom.c: tcd_eject exiting normally\n" );
@@ -426,12 +496,12 @@ int tcd_stopcd( cd_struct *cd )
 
 	cd->err = FALSE;
         tmp = ioctl( cd->cd_dev, CDROMSTOP );
-   	if( tmp < 0 )
+   	if(tmp)
 	{
 		strcpy( cd->errmsg, "Can't stop disc." );
                	cd->err = TRUE;
 
-		debug("cdrom.c: tcd_stopcd exiting early. CDROMSTOP ioctl error.\n" );
+		debug("cdrom.c: tcd_stopcd exiting early. CDROMSTOP ioctl error = %x.\n", tmp);
 		return(-1);
         }
 
@@ -447,7 +517,7 @@ int tcd_pausecd( cd_struct *cd )
 	if( cd->sc.cdsc_audiostatus==CDROM_AUDIO_PAUSED )
 	{       
 		tmp = ioctl( cd->cd_dev, CDROMRESUME );
-		if( tmp < 0 )
+		if(tmp)
 		{
 			strcpy( cd->errmsg, strerror( errno ) );
                         cd->err = TRUE;
@@ -458,7 +528,7 @@ int tcd_pausecd( cd_struct *cd )
 	else
 	{
 		tmp = ioctl( cd->cd_dev, CDROMPAUSE );
-		if( tmp < 0 )
+		if(tmp)
 		{
 			strcpy( cd->errmsg, strerror( errno ) );
                         cd->err = TRUE;
@@ -471,16 +541,13 @@ int tcd_pausecd( cd_struct *cd )
 int tcd_change_disc( cd_struct *cd, int disc )
 {
 #ifdef TCD_CHANGER_ENABLED
-	int tmp, fd;
+	int tmp;
 	cd->err = FALSE;
-	fd = open( cd->cdpath, O_RDONLY | O_NONBLOCK );
 
-        tmp = ioctl( fd, CDROM_SELECT_DISC, disc );
-	if( tmp < 0 )
+        tmp = ioctl( cd->cd_dev, CDROM_SELECT_DISC, disc );
+	if(tmp && errno)
 		fprintf( stdout, "ioctl: %s\n", strerror(errno) );	
-	/* Don't be noisy if there's an error */
 
-	close(fd);
    	return tmp;
 #else
 	debug("tcd_change_disc called, but changer support isn't compiled in. Ickyblah.\n" );
@@ -501,7 +568,7 @@ void tcd_opencddev( cd_struct *cd, WarnFunc msg_cb )
  
         if( cd->cd_dev < 0 )
         {
-		snprintf( tmp, 255, "Error accessing cdrom device.\n\
+		g_snprintf( tmp, 255, "Error accessing cdrom device.\n\
 			Please check to make sure cdrom drive support\n\
 			is compiled into the kernel, and that you have\n\
 			permission to access the device.\n\nReason: %s\n", strerror(errno));
@@ -558,7 +625,7 @@ int tcd_readdiskinfo( cd_struct *cd )
     strncat( tcd_dir, "/.cddbslave/", sizeof(tcd_dir) );
     tcd_dir[sizeof(tcd_dir) - 1] = 0;
     
-    snprintf( fn, sizeof(fn) - 1, "%s%08lx", tcd_dir, cd->cddb_id );
+    g_snprintf( fn, sizeof(fn) - 1, "%s%08lx", tcd_dir, cd->cddb_id );
 
     fp = fopen(fn, "r");	
     if(fp != NULL)
@@ -579,11 +646,11 @@ int tcd_readdiskinfo( cd_struct *cd )
 	/* Here's where we want to send a request to our slave */
 	tcd_call_cddb_slave(cd, "TCD", "3.0");
 	debug("Warning, can't open \'%s\' \n", fn );
-	strcpy( cd->dtitle, "Unknown / Unknown" );
+	strcpy( cd->dtitle, _("Unknown / Unknown") );
 	
 	for( i = cd->first_t; i <= cd->last_t; i++ )
 	{
-	    sprintf( cd->trk[C(i)].name, "Track %d", i );
+	    sprintf( cd->trk[C(i)].name, _("Track %d"), i );
 	    cd->trk[C(i)].titled = FALSE;
 	}
 	strcpy( cd->trk[0].name, "--" );
@@ -616,7 +683,7 @@ void tcd_writediskinfo( cd_struct *cd )
     
     strncpy( home, homedir, sizeof(home) );
     home[sizeof(home) - 1] = 0;
-    snprintf( fn, sizeof(fn) - 1, "%s/.cddbslave/%08lx", home, cd->cddb_id );
+    g_snprintf( fn, sizeof(fn) - 1, "%s/.cddbslave/%08lx", home, cd->cddb_id );
     
     if( tcd_writecddb(cd, fn) < 0 )
     {
