@@ -10,6 +10,12 @@ typedef enum
     CloseTray
 } TCDAction;
 
+typedef struct
+{
+    gint key;
+    gboolean ctrl, alt, shift;
+} Shortcut;
+
 typedef struct
 {
     gchar *cddev;
@@ -22,10 +28,19 @@ typedef struct
 
     TCDAction exit_action, start_action;
     gboolean close_tray_on_start;
+
+    int x,y,h,w;
+
+    Shortcut quit, play, stop, tracked;
+    Shortcut eject, back, forward;
+
+    gchar *cddb_server;
+    guint cddb_port;
 } tcd_prefs;
 
 void load_prefs( tcd_prefs *prop );
 void save_prefs( tcd_prefs *prop );
 void prefs_cb( GtkWidget *widget, void *data );
+void changed_cb(GtkWidget *widget, void *data);
 
 #endif
